<?php

namespace Drupal\layout_builder_boolean\Plugin\Derivative;

use Drupal\Component\Plugin\Derivative\DeriverBase;
use Drupal\Core\Layout\LayoutDefinition;
use Drupal\Core\Plugin\Context\EntityContextDefinition;
use Drupal\Core\Plugin\Discovery\YamlDiscovery;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\Layout\LayoutPluginManager;
use Drupal\Core\Plugin\Discovery\ContainerDeriverInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides LB Boolean version of all Layouts.
 *
 * @internal
 *   Plugin derivers are internal.
 */
class LayoutBuilderBooleanDeriver extends DeriverBase implements ContainerDeriverInterface {

  /**
   * The layout manager.
   *
   * @var \Drupal\Core\Layout\LayoutPluginManager
   */
  protected $layoutPluginManager;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The theme handler.
   *
   * @var \Drupal\Core\Extension\ThemeHandlerInterface
   */
  protected $themeHandler;

  /**
   *
   */
  public function __construct(LayoutPluginManager $layout_plugin_manager, ModuleHandlerInterface $module_handler, ThemeHandlerInterface $theme_handler) {
    $this->layoutPluginManager = $layout_plugin_manager;
    $this->moduleHandler = $module_handler;
    $this->themeHandler = $theme_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, $base_plugin_id) {
    return new static(
      $container->get('plugin.manager.core.layout'),
      $container->get('module_handler'),
      $container->get('theme_handler'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getDerivativeDefinitions($base_plugin_definition) {
    $layouts = $this->ymlLayoutDefinitions();
    foreach ($layouts as $layout_id => $definition) {
      $derivative = [];
      $derivative['class'] = '\Drupal\layout_builder_boolean\Plugin\Layout\LayoutBuilderBoolean';
      $derivative['id'] = 'layout_builder_boolean:' . $definition['id'];
      $derivative['theme_hook'] = 'layout__layout_builder_boolean';
      $derivative['library'] = 'layout_builder_boolean/layout_builder_boolean';

      $derivative['category'] = $definition['category'] ?? NULL;
      $derivative['icon_map'] = $definition['icon_map'] ?? [];
      $derivative['label'] = $definition['label'] . ' (Boolean)';
      foreach ($definition['regions'] as $key => $label) {
        $derivative['regions']["true:$key"] = $label;
        $derivative['regions']["false:$key"] = $label;
      }
      $context_definition = new EntityContextDefinition();
      $context_definition->setRequired(FALSE);
      $derivative['context_definitions'] = [
        'entity' => $context_definition,
      ];
      $this->derivatives[$layout_id] = new LayoutDefinition($derivative);
    }
    return $this->derivatives;
  }

  /**
   * Discover all layou plugins defined through yml.
   *
   * This is special discovery so that this derivative does not call
   * LayoutPluginManager::getDefinitions(). That would result in an
   * infinitely recursive nightmare.
   *
   * @return array
   *   An array of layout definition arrays, not objects.
   */
  protected function ymlLayoutDefinitions() {
    $discovery = new YamlDiscovery('layouts', $this->moduleHandler->getModuleDirectories() + $this->themeHandler->getThemeDirectories());
    $discovery
      ->addTranslatableProperty('label')
      ->addTranslatableProperty('description')
      ->addTranslatableProperty('category');
    return $discovery->getDefinitions();
  }

}
